@@ -1,4 +1,5 @@
 import random
+import re
 import shutil
 from pathlib import Path
 from subprocess import PIPE, call
@@ -22,12 +23,13 @@ from ape.exceptions import (
     VirtualMachineError,
 )
 from ape.logging import logger
-from ape.types import AddressType, SnapshotID
+from ape.types import AddressType, ContractCode, SnapshotID
 from ape.utils import cached_property
 from ape_test import Config as TestConfig
 from eth_utils import is_0x_prefixed, is_hex, to_hex
 from evm_trace import CallTreeNode, CallType, TraceFrame, get_calltree_from_geth_trace
 from hexbytes import HexBytes
+from pydantic import BaseModel, Field
 from web3 import HTTPProvider, Web3
 from web3.eth import TxParams
 from web3.exceptions import ExtraDataLengthError
@@ -39,8 +41,6 @@ from .exceptions import HardhatNotInstalledError, HardhatProviderError, HardhatS
 
 EPHEMERAL_PORTS_START = 49152
 EPHEMERAL_PORTS_END = 60999
-HARDHAT_START_NETWORK_RETRIES = [0.1, 0.2, 0.3, 0.5, 1.0]  # seconds between network retries
-HARDHAT_START_PROCESS_ATTEMPTS = 3  # number of attempts to start subprocess before giving up
 DEFAULT_PORT = 8545
 HARDHAT_CHAIN_ID = 31337
 HARDHAT_CONFIG = """
@@ -61,6 +61,7 @@ module.exports = {{
 }};
 """
 HARDHAT_CONFIG_FILE_NAME = "hardhat.config.js"
+HARDHAT_PLUGIN_PATTERN = re.compile(r"hardhat-[A-Za-z0-9-]+$")
 _NO_REASON_REVERT_MESSAGE = "Transaction reverted without a reason string"
 _REVERT_REASON_PREFIX = (
     "Error: VM Exception while processing transaction: reverted with reason string "
@@ -99,17 +100,22 @@ class HardhatConfigJS:
             self._path.write_text(self._content)
 
 
+class PackageJson(BaseModel):
+    name: Optional[str]
+    version: Optional[str]
+    description: Optional[str]
+    dependencies: Optional[Dict[str, str]]
+    dev_dependencies: Optional[Dict[str, str]] = Field(alias="devDependencies")
+
+
 class HardhatForkConfig(PluginConfig):
     upstream_provider: Optional[str] = None
     block_number: Optional[int] = None
+    enable_hardhat_deployments: bool = False
 
 
 class HardhatNetworkConfig(PluginConfig):
     port: Optional[Union[int, Literal["auto"]]] = DEFAULT_PORT
-
-    # Retry strategy configs, try increasing these if you're getting HardhatSubprocessError
-    network_retries: List[float] = HARDHAT_START_NETWORK_RETRIES
-    process_attempts: int = HARDHAT_START_PROCESS_ATTEMPTS
     request_timeout: int = 30
     fork_request_timeout: int = 300
 
@@ -145,7 +151,7 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
 
     @property
     def timeout(self) -> int:
-        return self.config.request_timeout  # type: ignore
+        return self.config.request_timeout
 
     @property
     def chain_id(self) -> int:
@@ -166,7 +172,7 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
             raise HardhatSubprocessError(
                 "NPM executable returned error code. See ape-hardhat README for install steps."
             )
-        elif _call(npx, "hardhat") != 0:
+        elif _call(npx, "hardhat", "--version") != 0:
             raise HardhatNotInstalledError()
 
         return npx
@@ -198,6 +204,33 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
     def _test_config(self) -> TestConfig:
         return cast(TestConfig, self.config_manager.get_config("test"))
 
+    @cached_property
+    def _package_json(self) -> PackageJson:
+        json_path = self.project_folder / "package.json"
+
+        if not json_path.is_file():
+            return PackageJson()
+
+        return PackageJson.parse_file(json_path)
+
+    @cached_property
+    def _hardhat_plugins(self) -> List[str]:
+        plugins: List[str] = []
+
+        def package_is_plugin(package: str) -> bool:
+            return re.search(HARDHAT_PLUGIN_PATTERN, package) is not None
+
+        if self._package_json.dependencies:
+            plugins.extend(filter(package_is_plugin, self._package_json.dependencies.keys()))
+
+        if self._package_json.dev_dependencies:
+            plugins.extend(filter(package_is_plugin, self._package_json.dev_dependencies.keys()))
+
+        return plugins
+
+    def _has_hardhat_plugin(self, plugin_name: str) -> bool:
+        return next((True for plugin in self._hardhat_plugins if plugin == plugin_name), False)
+
     def connect(self):
         """
         Start the hardhat process and verify it's up and accepting connections.
@@ -208,7 +241,7 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
 
         # NOTE: Must set port before calling 'super().connect()'.
         if not self.port:
-            self.port = self.config.port  # type: ignore
+            self.port = self.provider_settings.get("port", self.config.port)
 
         if self.is_connected:
             # Connects to already running process
@@ -227,7 +260,7 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
                         f"Connecting to existing '{self.process_name}' at port '{self.port}'."
                     )
             else:
-                for _ in range(self.config.process_attempts):  # type: ignore
+                for _ in range(self.config.process_attempts):
                     try:
                         self._start()
                         break
@@ -251,7 +284,8 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
 
         # Verify is actually a Hardhat provider,
         # or else skip it to possibly try another port.
-        client_version = self._web3.clientVersion
+        # TODO: Once we are on web3.py 0.6.0b8 or later, can just use snake_case here.
+        client_version = getattr(self._web3, "client_version", getattr(self._web3, "clientVersion"))
 
         if "hardhat" in client_version.lower():
             self._web3.eth.set_gas_price_strategy(rpc_gas_price_strategy)
@@ -261,19 +295,17 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
             )
 
         # Handle if using PoA Hardhat
-
-        def began_poa() -> bool:
-            try:
-                block = self.web3.eth.get_block(0)
-            except ExtraDataLengthError:
-                return True
-
-            return (
+        try:
+            block = self.web3.eth.get_block(0)
+        except ExtraDataLengthError:
+            began_poa = True
+        else:
+            began_poa = (
                 "proofOfAuthorityData" in block
                 or len(block.get("extraData", "")) > MAX_EXTRADATA_LENGTH
             )
 
-        if began_poa():
+        if began_poa:
             self._web3.middleware_onion.inject(geth_poa_middleware, layer=0)
 
     def _start(self):
@@ -320,7 +352,7 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
     def set_block_gas_limit(self, gas_limit: int) -> bool:
         return self._make_request("evm_setBlockGasLimit", [hex(gas_limit)]) is True
 
-    def set_code(self, address: AddressType, code: Union[str, bytes, HexBytes]) -> bool:
+    def set_code(self, address: AddressType, code: ContractCode) -> bool:
         if isinstance(code, bytes):
             code = code.hex()
 
@@ -367,9 +399,9 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
             # Allow for an unsigned transaction
             txn = self.prepare_transaction(txn)
             txn_dict = txn.dict()
+            txn_params = cast(TxParams, txn_dict)
 
             try:
-                txn_params = cast(TxParams, txn_dict)
                 txn_hash = self.web3.eth.send_transaction(txn_params)
             except ValueError as err:
                 raise self.get_virtual_machine_error(err) from err
@@ -392,20 +424,26 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
 
     def get_call_tree(self, txn_hash: str) -> CallTreeNode:
         receipt = self.chain_manager.get_receipt(txn_hash)
-        root_node_kwargs = {
-            "gas_cost": receipt.gas_used,
-            "gas_limit": receipt.gas_limit,
-            "address": receipt.receiver,
-            "calldata": receipt.data,
-            "value": receipt.value,
-            "call_type": CallType.CALL,
-            "failed": receipt.failed,
-        }
-        return get_calltree_from_geth_trace(receipt.trace, **root_node_kwargs)
+
+        # Subtract base gas costs.
+        # (21_000 + 4 gas per 0-byte and 16 gas per non-zero byte).
+        data_gas = sum([4 if x == 0 else 16 for x in receipt.data])
+        method_gas_cost = receipt.gas_used - 21_000 - data_gas
+
+        return get_calltree_from_geth_trace(
+            receipt.trace,
+            gas_cost=method_gas_cost,
+            gas_limit=receipt.gas_limit,
+            address=receipt.receiver,
+            calldata=receipt.data,
+            value=receipt.value,
+            call_type=CallType.CALL,
+            failed=receipt.failed,
+        )
 
     def set_balance(self, account: AddressType, amount: Union[int, float, str, bytes]):
         is_str = isinstance(amount, str)
-        is_hex = False if not is_str else is_0x_prefixed(str(amount))
+        _is_hex = False if not is_str else is_0x_prefixed(str(amount))
         is_key_word = is_str and len(str(amount).split(" ")) > 1
         if is_key_word:
             # This allows values such as "1000 ETH".
@@ -415,7 +453,7 @@ class HardhatProvider(SubprocessProvider, Web3Provider, TestProviderAPI):
         amount_hex_str = str(amount)
 
         # Convert to hex str
-        if is_str and not is_hex:
+        if is_str and not _is_hex:
             amount_hex_str = to_hex(int(amount))
         elif isinstance(amount, int) or isinstance(amount, bytes):
             amount_hex_str = to_hex(amount)
@@ -470,9 +508,13 @@ class HardhatForkProvider(HardhatProvider):
     def fork_block_number(self) -> Optional[int]:
         return self._fork_config.block_number
 
+    @property
+    def enable_hardhat_deployments(self) -> bool:
+        return self._fork_config.enable_hardhat_deployments
+
     @property
     def timeout(self) -> int:
-        return self.config.fork_request_timeout  # type: ignore
+        return self.config.fork_request_timeout
 
     @property
     def _upstream_network_name(self) -> str:
@@ -537,6 +579,10 @@ class HardhatForkProvider(HardhatProvider):
 
         cmd = super().build_command()
         cmd.extend(("--fork", self.fork_url))
+
+        # --no-deploy option is only available if hardhat-deploy is installed
+        if not self.enable_hardhat_deployments and self._has_hardhat_plugin("hardhat-deploy"):
+            cmd.append("--no-deploy")
         if self.fork_block_number is not None:
             cmd.extend(("--fork-block-number", str(self.fork_block_number)))
 
