@@ -8,21 +8,21 @@ from evm_trace import CallTreeNode, CallType, TraceFrame
 from hexbytes import HexBytes
 
 from ape_hardhat.exceptions import HardhatProviderError
-from ape_hardhat.provider import HARDHAT_CHAIN_ID, HARDHAT_CONFIG_FILE_NAME, HardhatProvider
+from ape_hardhat.provider import HARDHAT_CHAIN_ID, HARDHAT_CONFIG_FILE_NAME
 
 TEST_WALLET_ADDRESS = "0xD9b7fdb3FC0A0Aa3A507dCf0976bc23D49a9C7A3"
 
 
-def test_instantiation(hardhat_disconnected):
-    assert hardhat_disconnected.name == "hardhat"
+def test_instantiation(disconnected_provider):
+    assert disconnected_provider.name == "hardhat"
 
 
 def test_connect_and_disconnect(create_provider):
     # Use custom port to prevent connecting to a port used in another test.
 
-    hardhat = create_provider()
-    hardhat.port = 8555
-    hardhat.connect()
+    provider = create_provider()
+    provider.port = 8555
+    provider.connect()
 
     # Verify config file got created
     config = Path(HARDHAT_CONFIG_FILE_NAME)
@@ -31,13 +31,13 @@ def test_connect_and_disconnect(create_provider):
     assert DEFAULT_TEST_MNEMONIC in config_text
 
     try:
-        assert hardhat.is_connected
-        assert hardhat.chain_id == HARDHAT_CHAIN_ID
+        assert provider.is_connected
+        assert provider.chain_id == HARDHAT_CHAIN_ID
     finally:
-        hardhat.disconnect()
+        provider.disconnect()
 
-    assert not hardhat.is_connected
-    assert hardhat.process is None
+    assert not provider.is_connected
+    assert provider.process is None
 
 
 def test_gas_price(connected_provider):
@@ -45,9 +45,9 @@ def test_gas_price(connected_provider):
     assert gas_price > 1
 
 
-def test_uri_disconnected(hardhat_disconnected):
+def test_uri_disconnected(disconnected_provider):
     with pytest.raises(HardhatProviderError) as err:
-        _ = hardhat_disconnected.uri
+        _ = disconnected_provider.uri
 
     assert "Can't build URI before `connect()` is called." in str(err.value)
 
@@ -57,19 +57,7 @@ def test_uri(connected_provider):
     assert expected_uri in connected_provider.uri
 
 
-@pytest.mark.parametrize(
-    "method,args,expected",
-    [
-        (HardhatProvider.get_nonce, [TEST_WALLET_ADDRESS], 0),
-        (HardhatProvider.get_balance, [TEST_WALLET_ADDRESS], 0),
-        (HardhatProvider.get_code, [TEST_WALLET_ADDRESS], HexBytes("")),
-    ],
-)
-def test_rpc_methods(connected_provider, method, args, expected):
-    assert method(connected_provider, *args) == expected
-
-
-def test_multiple_hardhat_instances(create_provider):
+def test_multiple_instances(create_provider):
     """
     Validate the somewhat tricky internal logic of running multiple Hardhat subprocesses
     under a single parent process.
@@ -137,6 +125,10 @@ def test_revert_failure(connected_provider):
     assert connected_provider.revert(0xFFFF) is False
 
 
+def test_get_balance(connected_provider, owner):
+    assert connected_provider.get_balance(owner.address)
+
+
 def test_snapshot_and_revert(connected_provider):
     snap = connected_provider.snapshot()
 
@@ -169,11 +161,11 @@ def test_get_call_tree(connected_provider, sender, receiver):
     call_tree = connected_provider.get_call_tree(transfer.txn_hash)
     assert isinstance(call_tree, CallTreeNode)
     assert call_tree.call_type == CallType.CALL
-    assert repr(call_tree) == "CALL: 0xc89D42189f0450C2b2c3c61f58Ec5d628176A1E7 [21000 gas]"
+    assert repr(call_tree) == "CALL: 0xc89D42189f0450C2b2c3c61f58Ec5d628176A1E7 [0 gas]"
 
 
 def test_request_timeout(connected_provider, config, create_provider):
-    actual = connected_provider.web3.provider._request_kwargs["timeout"]  # type: ignore
+    actual = connected_provider.web3.provider._request_kwargs["timeout"]
     expected = 29  # Value set in `ape-config.yaml`
     assert actual == expected
 
@@ -212,7 +204,7 @@ def test_transaction_contract_as_sender(contract_instance, connected_provider):
 @pytest.mark.parametrize(
     "amount", ("50 ETH", int(50e18), "0x2b5e3af16b1880000", "50000000000000000000")
 )
-def test_set_account_balance(connected_provider, owner, convert, amount):
+def test_set_balance(connected_provider, owner, convert, amount):
     connected_provider.set_balance(owner.address, amount)
     assert owner.balance == convert("50 ETH", int)
 
